package org.greenstone.gsdl3;

import org.greenstone.gsdl3.core.*;
import org.greenstone.gsdl3.util.*;
import org.greenstone.gsdl3.action.PageAction; // used to get the default action
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.Enumeration;
import java.io.File;

/** a servlet to serve the greenstone library - we are using servlets instead
 * of cgi
 * the init method is called only once - the first time the servlet classes
 * are loaded. Each time a request comes in to the servlet, the session()
 * method is called in a new thread (calls doGet/doPut etc)
 * takes the a=p&p=home type args and builds a simple request to send to 
 * its receptionist, which returns a result in html, cos output=html
 * is set in the request
 * @see Receptionist
 */
public class LibraryServlet extends HttpServlet {
    
    /** the receptionist to send messages to */
    protected Receptionist recept_=null;
    /** the default language - is specified by setting a servlet param,
     * otherwise DEFUALT_LANG is used*/
    protected String default_lang_= null;
    /** The default default - used if a default lang is not specified 
     * in the servlet params */
    protected final String DEFAULT_LANG = "en";
    /** container Document to create XML Nodes */
    protected Document doc_=null;    
    /** a converter class to parse XML and create Docs */
    protected XMLConverter converter_=null;

    /** initialise the servlet
     */
    public void init(ServletConfig config) throws ServletException {
	// always call super.init;
	super.init(config);

	ConfigVars config_vars = new ConfigVars();
	String library_name = config.getInitParameter("libraryname");
	String gsdl3_home = config.getInitParameter("gsdl3home");
	String site_name = config.getInitParameter("sitename");
	String interface_name = config.getInitParameter("interfacename");
	default_lang_ = config.getInitParameter("defaultlang");
	if (library_name == null || gsdl3_home == null || site_name ==null ||  interface_name ==null) {
	    // must have this
	    System.err.println("initialisation parameters not all set!");
	    System.err.println(" you must have libraryname, gsdl3home, sitename,  interfacename");
	    System.exit(1);
	}

	if (default_lang_ == null) {
	    // choose english
	    default_lang_ = DEFAULT_LANG;
	}

	config_vars.library_name_ = library_name;
	config_vars.gsdl3_home_ = gsdl3_home;
	config_vars.site_name_ = site_name;
	config_vars.interface_name_ =interface_name;
	config_vars.default_language_ = default_lang_;
	config_vars.createXML();
	
	// new message router - create it and pass reference to recept.
	// the servlet wont use this directly
	MessageRouter message_router = new MessageRouter();
	message_router.setSiteHome(GSFile.siteHome(config_vars.gsdl3_home_, 
						    config_vars.site_name_));
	message_router.configure();

	// the receptionist -the servlet will talk to this
	recept_ = new Receptionist();
	recept_.setConfigVars(config_vars);
	recept_.setMessageRouter(message_router);
	recept_.configure();

	converter_ = new XMLConverter();
	doc_ = converter_.newDOM();

    }
    
    public void doGet(HttpServletRequest request,
		      HttpServletResponse response)
	throws ServletException, IOException {
	
	request.setCharacterEncoding("UTF-8");
	response.setContentType("text/html;charset=UTF-8");
	PrintWriter out = response.getWriter();

	String lang = request.getParameter(GSCGI.LANGUAGE_ARG);
	if (lang==null || lang.equals("")) {
	    lang = default_lang_; // the default
	}
	String output = request.getParameter(GSCGI.OUTPUT_ARG);
	if (output==null || output.equals("")) {
	    output = "html"; // uses html by default
	}

	// the request to the receptionist	
	Element xml_message = doc_.createElement(GSXML.MESSAGE_ELEM);
	Element xml_request = doc_.createElement(GSXML.REQUEST_ELEM);
	xml_message.appendChild(xml_request);
	xml_request.setAttribute(GSXML.LANG_ATT, lang);
	xml_request.setAttribute(GSXML.TYPE_ATT, GSXML.REQUEST_TYPE_CGI);
	xml_request.setAttribute(GSXML.OUTPUT_ATT, output);
	
	String action = request.getParameter(GSCGI.ACTION_ARG);
	String subaction = request.getParameter(GSCGI.SUBACTION_ARG);
	if (action==null || action.equals("")) {
	    // should we do all the following stuff if using default page?
	    // display the home page  - the default page
	    action = "p";
	    subaction = PageAction.HOME_PAGE;
	    xml_request.setAttribute(GSXML.ACTION_ATT, action);
	    xml_request.setAttribute(GSXML.SUBACTION_ATT, subaction);
	    
	    Element xml_result = recept_.process(xml_message);
	    out.println(converter_.getString(xml_result));
	    return;
	} 
	
	xml_request.setAttribute(GSXML.ACTION_ATT, action);
	if (subaction != null) {
	    xml_request.setAttribute(GSXML.SUBACTION_ATT, subaction);
	}
	Element xml_param_list = doc_.createElement(GSXML.PARAM_ELEM+GSXML.LIST_MODIFIER);
	xml_request.appendChild(xml_param_list);
	Enumeration params = request.getParameterNames();
	while(params.hasMoreElements()) {
	    String name = (String)params.nextElement();
	    if (!name.equals(GSCGI.ACTION_ARG) && !name.equals(GSCGI.SUBACTION_ARG) && !name.equals(GSCGI.LANGUAGE_ARG) && !name.equals(GSCGI.OUTPUT_ARG)) { // these ones are in the header so leave out
		String value="";
		String []values = request.getParameterValues(name);
		value = values[0];
		if (values.length > 1) {
		    for (int i=1; i< values.length; i++) {
			value += ","+values[i];
		    }
		}
		Element param = doc_.createElement(GSXML.PARAM_ELEM);
		param.setAttribute(GSXML.NAME_ATT, name);
		param.setAttribute(GSXML.VALUE_ATT, GSXML.xmlSafe(value));
		xml_param_list.appendChild(param);
		
	    }
	}

	if (!output.equals("html")) {
	    response.setContentType("text/xml"); // for now use text
	}
    
	Element xml_result = recept_.process(xml_message);
	out.println(converter_.getPrettyString(xml_result));
    }
}
